<?php
/**
 * Plugin Name: DMG Query Loop Block Enhancements
 * Plugin URI:  https://github.com/ANMedia/dmg-query-loop-block-enhancements-wp-plugin
 * Description: Enhancements for the core Query Loop block
 * Version:     1.1.0
 * Author:      Metro.co.uk
 * Author URI:  https://github.com/ANMedia/dmg-query-loop-block-enhancements-wp-plugin/graphs/contributors
 */

namespace DMG;

/**
 * Class Query_Loop_Block_Enhancements
 *
 * Register all scripts necessary for query loop customizations and add filters
 *
 * @subpackage DMG
 */
class Query_Loop_Block_Enhancements {

	/**
	 * Init
	 */
	public static function init() {
		add_action( 'init', array( __CLASS__, 'filter_block_patterns' ), 99 );
		add_action( 'enqueue_block_editor_assets', array( __CLASS__, 'enqueue_assets' ) );
		add_filter( 'pre_render_block', array( __CLASS__, 'pre_render_post_curator_block' ), 10, 2 );
		add_action( 'pre_get_posts', array( __CLASS__, 'filter_multi_post_types_query_var' ) );
	}

	/**
	 * Check the WP_Block_Patterns_Registry single to see which block patterns
	 * are registered to the core/query block. Unregister them and assign our
	 * new blocks to them so that they can be used in the pattern selector when
	 * the user is choosing a template.
	 * 
	 * Core does not offer many (any?) hooks for managing patterns, so this
	 * is our best attempt at instituting a "filter".
	 */
	public static function filter_block_patterns() {

		// Get all registered patterns
		$patterns = \WP_Block_Patterns_Registry::get_instance()->get_all_registered();

		// Filter only patterns assigned to core/query
		$filtered_patterns = array_filter(
			$patterns,
			function( $pattern ) {
				return ( isset( $pattern['blockTypes'] ) && in_array( 'core/query', $pattern['blockTypes'], true ) );
			}
		);

		foreach ( $filtered_patterns as $pattern ) {

			// Add in our custom variation as an acceptable block type.
			// IMPORTANT: Note the core/query prefix, which is required
			// since this is a variation for the core/query block
			$pattern['blockTypes'] = [ 'core/query/dmg/query-loop-block-variations/post-curator' ];

			// Rewrite the block attributes to add in the namespace into the query attributes
			preg_match( '/(<!-- wp:query(.+)-->)/', $pattern['content'], $matches );

			if ( ! empty( $matches[1] ) ) {

				// New array of attributes that just contains the namespace
				$new_attributes = array(
					'namespace' => 'dmg/query-loop-block-variations/post-curator'
				);

				// JSON-encode the new attributes
				$new_attributes_json = sprintf(
					'<!-- wp:query %s -->',
					wp_json_encode( $new_attributes )
				);

				// String replace old attributes with the new
				$pattern['content'] = str_replace( $matches[1], $new_attributes_json, $pattern['content'] );
			}

			// Prefix the pattern to give it a unique name
			$pattern['name'] = 'dmg/query-loop-block-variations/post-curator/' . $pattern['name'];

			// Register the new variation
			register_block_pattern( $pattern['name'], $pattern );
		}

	}

	/**
	 * Enqueue block editor assets
	 */
	public static function enqueue_assets() {
		$handle = 'dmg-query-loop-block-enhancements';

		// Include asset file for dependencies
		$asset_file = include( plugin_dir_path( __FILE__ ) . 'build/index.asset.php');

		wp_enqueue_script(
			$handle,
			plugins_url( 'build/index.js', __FILE__ ),
			$asset_file['dependencies'],
			$asset_file['version']
		);

		wp_enqueue_style(
			$handle,
			plugins_url( 'build/index.css', __FILE__ ),
			array(),
			$asset_file['version']
		);
	}

	/**
	 * Conditionally filter the query loop vars when we are rendering
	 * the post curator block.
	 * 
	 * This return returns null and isn't intended to actually modify
	 * any values. We can use it as an action by continuing to return
	 * null and adding a filter.
	 * 
	 * @param string|null $pre_render   The default null value
	 * @param array       $parsed_block The parsed block
	 * @return null Return null to allow the block to render
	 */
	public static function pre_render_post_curator_block( $pre_render, $parsed_block ) {

		// Use namespace to determine if we should add the filter
		if ( 'dmg/query-loop-block-variations/post-curator' === $parsed_block['attrs']['namespace'] ) {
			add_filter( 'query_loop_block_query_vars', array( __CLASS__, 'filter_block_query' ), 10, 2 );
		}
	}

	/**
	 * Check for curated posts and configure the query accordingly
	 *
	 * @param array    $query Query arguments
	 * @param WP_Block $block The block instance
	 * @return array The Updated query arguments
	 */
	public static function filter_block_query( $query, $block ) {

		remove_filter( 'query_loop_block_query_vars', array( __CLASS__, 'filter_block_query' ), 10, 2 );

		// Check the include property
		$curated_posts = ( ! empty( $block->context['query']['include'] ) ) ? $block->context['query']['include'] : '';

		// Separate into post IDs
		$post_ids = explode( ',', $curated_posts );

		if ( empty( $post_ids ) ) {
			return $query;
		}

		// Set proper query vars
		$query['orderby']        = 'post__in';
		$query['post__in']       = $post_ids;
		$query['posts_per_page'] = count( $post_ids );
		$query['post_type']      = 'any';

		return $query;
	}

	/**
	 * Handling for query vars indicating selection of all post types
	 * 
	 * @param WP_Query $query The query object
	 */
	public static function filter_multi_post_types_query_var( $query ) {

		// Check for query var and set post type to any
		if ( ! empty( $_GET['dmg_query_loop_block_enhancements_post_curator'] ) ) {
			$query->set( 'post_type', 'any' );
			$query->set( 'post_status', array( 'publish', 'future' ) );
		}

		return $query;
	}
}

Query_Loop_Block_Enhancements::init();
